
#
# A simple class to demonstrate Python special methods
#
# author: David Mathias
#

class Account():

    # a class variable -- shared by all instances
    num_accounts = 0
    
    # create instance variables and initialize values        
    def __init__(self, name='unassigned', balance=0):
        self.name = name
        self.balance = balance
        Account.num_accounts += 1
    
    # do this when reference count for an instance reaches 0
    def __del__(self):
        Account.num_accounts -= 1
        
    def __str__(self):
        s = 'Account holder:  ' + self.name + '\n'
        s += 'Account balance: ' + str(self.balance) + '\n'
        return s
    
    def __repr__(self):
        # f is a form of string formatting available only in Python 3
        # !r tells Python to use the representation of the attribute
        return f'Account({self.name!r}, {self.balance!r})'
    
    def __eq__(self, other):
        if self.name == other.name and self.balance == other.balance:
            return True
        return False
    
    def __lt__(self, other):
        if self.balance < other.balance:
            return True
        elif self.balance == other.balance and self.name < other.name:
            return True
        else:
            return False
        
    def __add__(self, other):
        return self.balance + other.balance
    
    def __iadd__(self, other):
        self.balance += other.balance
        return self
    
    def __bool__(self):
        return len(self.name) > 0 and self.name != 'unassigned'
        
    def __call__(self, dep):
        self.balance += dep
        
    # these are the instance methods
    def deposit(self, amt):
        self.balance += amt
        
    def withdraw(self, amt):
        self.balance -= amt
        
    def inquiry(self):
        return self.balance
    